@@ -360,7 +360,7 @@ public:
 #endif
 
 	// Allocate memory block. Result is not zero-initialized.
-	// It case of problems this method throws Firebird::BadAlloc
+	// In case of problems this method throws Firebird::BadAlloc
 	void* allocate(size_t size
 #ifdef DEBUG_GDS_ALLOC
 		, const char* file = NULL, int line = 0
@@ -376,6 +376,13 @@ public:
 
 	void deallocate(void* block);
 
+	// Allocate huge memory block directly from OS. 
+	// In case of problems this method throws Firebird::BadAlloc
+	void* allocateHugeBlock(size_t size);
+
+	// Return huge memory block to the OS directly. 
+	void deallocateHugeBlock(void* block, size_t size);
+
 	// Check pool for internal consistent. When enabled, call is very expensive
 	bool verify_pool(bool fast_checks_only = false);
 
@@ -492,23 +499,11 @@ using Firebird::MemoryPool;
 inline static MemoryPool* getDefaultMemoryPool() { return Firebird::MemoryPool::processMemoryPool; }
 
 // Global versions of operators new and delete
-inline void* operator new(size_t s) THROW_BAD_ALLOC
-{
-	return Firebird::MemoryPool::globalAlloc(s);
-}
-inline void* operator new[](size_t s) THROW_BAD_ALLOC
-{
-	return Firebird::MemoryPool::globalAlloc(s);
-}
+void* operator new(size_t s) THROW_BAD_ALLOC;
+void* operator new[](size_t s) THROW_BAD_ALLOC;
 
-inline void operator delete(void* mem) throw()
-{
-	Firebird::MemoryPool::globalFree(mem);
-}
-inline void operator delete[](void* mem) throw()
-{
-	Firebird::MemoryPool::globalFree(mem);
-}
+void operator delete(void* mem) throw();
+void operator delete[](void* mem) throw();
 
 #ifdef DEBUG_GDS_ALLOC
 inline void* operator new(size_t s, Firebird::MemoryPool& pool, const char* file, int line)
