@@ -91,7 +91,7 @@ static bool		get_indices(SLONG*, const UCHAR**, SLONG*, SCHAR**);
 static USHORT	get_request_info(thread_db*, dsql_req*, SLONG, UCHAR*);
 static bool		get_rsb_item(SLONG*, const UCHAR**, SLONG*, SCHAR**, USHORT*, USHORT*);
 static dsql_dbb*	init(Attachment*);
-static void		map_in_out(dsql_req*, dsql_msg*, USHORT, const UCHAR*, USHORT, UCHAR*, const UCHAR* = 0);
+static void		map_in_out(thread_db*, dsql_req*, dsql_msg*, USHORT, const UCHAR*, USHORT, UCHAR*, const UCHAR* = 0);
 static USHORT	parse_blr(USHORT, const UCHAR*, const USHORT, dsql_par*);
 static dsql_req*		prepare(thread_db*, dsql_dbb*, jrd_tra*, USHORT, const TEXT*, USHORT, USHORT);
 static UCHAR*	put_item(UCHAR, const USHORT, const UCHAR*, UCHAR*, const UCHAR* const, const bool copy = true);
@@ -372,6 +372,12 @@ ISC_STATUS DSQL_fetch(thread_db* tdbb,
 		}
 	}
 
+	if (!request->req_request)
+	{
+		ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-504) <<
+				  Arg::Gds(isc_unprepared_stmt));
+	}
+
 #ifdef SCROLLABLE_CURSORS
 
 	// check whether we need to send an asynchronous scrolling message
@@ -520,7 +526,7 @@ ISC_STATUS DSQL_fetch(thread_db* tdbb,
 		return 100;
 	}
 
-	map_in_out(NULL, message, 0, blr, msg_length, dsql_msg_buf);
+	map_in_out(tdbb, NULL, message, 0, blr, msg_length, dsql_msg_buf);
 
 	trace.fetch(false, res_successful);
 	return FB_SUCCESS;
@@ -1001,7 +1007,7 @@ static void execute_blob(thread_db* tdbb,
 	UCHAR bpb[24];
 
 	dsql_blb* blob = request->req_blob;
-	map_in_out(request, blob->blb_open_in_msg, in_blr_length, in_blr, in_msg_length, NULL, in_msg);
+	map_in_out(tdbb, request, blob->blb_open_in_msg, in_blr_length, in_blr, in_msg_length, NULL, in_msg);
 
 	UCHAR* p = bpb;
 	*p++ = isc_bpb_version1;
@@ -1046,7 +1052,7 @@ static void execute_blob(thread_db* tdbb,
 		request->req_blob->blb_blob =
 			BLB_create2(tdbb, request->req_transaction, blob_id, bpb_length, bpb);
 
-		map_in_out(NULL, blob->blb_open_out_msg, out_blr_length, out_blr, out_msg_length, out_msg);
+		map_in_out(tdbb, NULL, blob->blb_open_out_msg, out_blr_length, out_blr, out_msg_length, out_msg);
 	}
 }
 
@@ -1256,11 +1262,17 @@ static void execute_request(thread_db* tdbb,
 		fb_assert(false);
 	}
 
+	if (!request->req_request)
+	{
+		ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-504) <<
+				  Arg::Gds(isc_unprepared_stmt));
+	}
+
 	// If there is no data required, just start the request
 
 	dsql_msg* message = request->req_send;
 	if (message)
-		map_in_out(request, message, in_blr_length, in_blr, in_msg_length, NULL, in_msg);
+		map_in_out(tdbb, request, message, in_blr_length, in_blr, in_msg_length, NULL, in_msg);
 
 	// we need to map_in_out before tracing of execution start to let trace
 	// manager know statement parameters values
@@ -1278,10 +1290,7 @@ static void execute_request(thread_db* tdbb,
 	// Selectable execute block should get the "proc fetch" flag assigned,
 	// which ensures that the savepoint stack is preserved while suspending
 	if (request->req_type == REQ_SELECT_BLOCK)
-	{
-		fb_assert(request->req_request);
 		request->req_request->req_flags |= req_proc_fetch;
-	}
 
 	// REQ_EXEC_BLOCK has no outputs so there are no out_msg
 	// supplied from client side, but REQ_EXEC_BLOCK requires
@@ -1311,7 +1320,7 @@ static void execute_request(thread_db* tdbb,
 			message->msg_buffer, 0);
 
 		if (out_msg_length)
-			map_in_out(NULL, message, 0, out_blr, out_msg_length, out_msg);
+			map_in_out(tdbb, NULL, message, 0, out_blr, out_msg_length, out_msg);
 
 		// if this is a singleton select, make sure there's in fact one record
 
@@ -2168,7 +2177,8 @@ static dsql_dbb* init(Attachment* attachment)
     @param in_dsql_msg_buf
 
  **/
-static void map_in_out(	dsql_req*		request,
+static void map_in_out(	thread_db*		tdbb,
+						dsql_req*		request,
 						dsql_msg*		message,
 						USHORT	blr_length,
 						const UCHAR*	blr,
@@ -2176,12 +2186,18 @@ static void map_in_out(	dsql_req*		request,
 						UCHAR*	dsql_msg_buf,
 						const UCHAR* in_dsql_msg_buf)
 {
-	thread_db* tdbb = JRD_get_thread_data();
+	// When mapping data from the external world, request will be non-NULL.
+	// When mapping data from an internal message, request will be NULL.
 
 	USHORT count = parse_blr(blr_length, blr, msg_length, message->msg_parameters);
 
-	// When mapping data from the external world, request will be non-NULL.
-	// When mapping data from an internal message, request will be NULL.
+	// Sanity check
+
+	if (count && !(request ? in_dsql_msg_buf : dsql_msg_buf))
+	{
+		ERRD_post(Arg::Gds(isc_sqlerr) << Arg::Num(-804) <<
+				  Arg::Gds(isc_dsql_sqlda_err));
+	}
 
 	dsql_par* parameter;
 
@@ -3251,16 +3267,29 @@ static UCHAR* var_info(dsql_msg* message,
 				break;
 
 			case dtype_text:
-				if (input_message && (param->par_desc.dsc_flags & DSC_null))
+			case dtype_varying:
+				if (input_message &&
+					(param->par_desc.dsc_dtype == dtype_text || param->par_is_text) &&
+					(param->par_desc.dsc_flags & DSC_null))
 				{
 					sql_type = SQL_NULL;
 					sql_len = 0;
+					sql_sub_type = 0;
 				}
-				else
+				else if (param->par_desc.dsc_dtype == dtype_text || param->par_is_text)
 				{
 					sql_type = SQL_TEXT;
 					sql_sub_type = param->par_desc.dsc_sub_type;
+					if (param->par_desc.dsc_dtype == dtype_varying)
+						sql_len -= sizeof(USHORT);
 				}
+				else
+				{
+					sql_type = SQL_VARYING;
+					sql_len -= sizeof(USHORT);
+					sql_sub_type = param->par_desc.dsc_sub_type;
+				}
+
 				break;
 
 			case dtype_blob:
@@ -3269,12 +3298,6 @@ static UCHAR* var_info(dsql_msg* message,
 				sql_scale = param->par_desc.dsc_scale;
 				break;
 
-			case dtype_varying:
-				sql_type = SQL_VARYING;
-				sql_len -= sizeof(USHORT);
-				sql_sub_type = param->par_desc.dsc_sub_type;
-				break;
-
 			case dtype_short:
 			case dtype_long:
 			case dtype_int64:
