@@ -255,7 +255,8 @@ blb* BLB_create(thread_db* tdbb, jrd_tra* transaction, bid* blob_id)
 
 blb* BLB_create2(thread_db* tdbb,
 				jrd_tra* transaction, bid* blob_id,
-				USHORT bpb_length, const UCHAR* bpb)
+				USHORT bpb_length, const UCHAR* bpb,
+				bool userBlob)
 {
 /**************************************
  *
@@ -353,6 +354,22 @@ blb* BLB_create2(thread_db* tdbb,
 	page->blp_header.pag_type = pag_blob;
 	blob->blb_segment = (UCHAR *) page->blp_page;
 
+	// Bind non-user blobs to the request
+
+	jrd_req* request = tdbb->getRequest();
+	if (!userBlob && request)
+	{
+		transaction->tra_blobs->locate(blob->blb_temp_id);
+		BlobIndex* current = &transaction->tra_blobs->current();
+
+		jrd_req* blob_request = request;
+		while (blob_request->req_caller)
+			blob_request = blob_request->req_caller;
+
+		current->bli_request = blob_request;
+		current->bli_request->req_blobs.add(blob->blb_temp_id);
+	}
+
 	// Format blob id and return blob handle
 
 	blob_id->set_temporary(blob->blb_temp_id);
@@ -1003,7 +1020,11 @@ void BLB_move(thread_db* tdbb, dsc* from_desc, dsc* to_desc, jrd_nod* field)
 		}
 	}
 
-	bid* source = (bid*) from_desc->dsc_address;
+	// Use local copy of source blob id to not change contents of from_desc in 
+	// a case when it points to materialized temporary blob (see below for 
+	// assignment to *source).
+	bid srcBlobID = *(bid*) from_desc->dsc_address;
+	bid* source = &srcBlobID;
 	bid* destination = (bid*) to_desc->dsc_address;
 
 	// If nothing changed, do nothing.  If it isn't broken, don't fix it.
@@ -1134,8 +1155,10 @@ void BLB_move(thread_db* tdbb, dsc* from_desc, dsc* to_desc, jrd_nod* field)
 			if (transaction->tra_blobs->locate(source->bid_temp_id()))
 			{
 				blobIndex = &transaction->tra_blobs->current();
+
 				if (blobIndex->bli_materialized)
 				{
+					/*** ASF: This sanity check is not correct for EXECUTE STATEMENT. See CORE-4752.
 					if (blobIndex->bli_request)
 					{
 						// Walk through call stack looking if our BLOB is
@@ -1146,13 +1169,16 @@ void BLB_move(thread_db* tdbb, dsc* from_desc, dsc* to_desc, jrd_nod* field)
 								break;
 							temp_req = temp_req->req_caller;
 						} while (temp_req);
+
 						if (!temp_req)
 						{
 							// Trying to use temporary id of materialized blob from another request
 							ERR_post(Arg::Gds(isc_bad_segstr_id));
 						}
 					}
-					source = &blobIndex->bli_blob_id;
+					***/
+
+					*source = blobIndex->bli_blob_id;
 					continue;
 				}
 
@@ -1281,6 +1307,10 @@ blb* BLB_open2(thread_db* tdbb,
 	SET_TDBB(tdbb);
 	Database* dbb = tdbb->getDatabase();
 
+	// Use local copy of input blob id to not change contents of *blob_id in a 
+	// case when it points to materialized temporary blob.
+	bid blobId = *blob_id;
+
 	// Handle filter case
 	SSHORT from, to;
 	SSHORT from_charset, to_charset;
@@ -1303,9 +1333,9 @@ blb* BLB_open2(thread_db* tdbb,
 	bool try_relations = false;
 	BlobIndex* current = NULL;
 
-	if (!blob_id->bid_internal.bid_relation_id)
+	if (!blobId.bid_internal.bid_relation_id)
 	{
-		if (blob_id->isEmpty())
+		if (blobId.isEmpty())
 			blob->blb_flags |= BLB_eof;
 		else
 		{
@@ -1319,13 +1349,13 @@ blb* BLB_open2(thread_db* tdbb,
 
 			// Search the index of transaction blobs for a match
 			const blb* new_blob = NULL;
-			if (transaction->tra_blobs->locate(blob_id->bid_temp_id()))
+			if (transaction->tra_blobs->locate(blobId.bid_temp_id()))
 			{
 				current = &transaction->tra_blobs->current();
 				if (!current->bli_materialized)
 					new_blob = current->bli_blob_object;
 				else
-					blob_id = &current->bli_blob_id;
+					blobId = current->bli_blob_id;
 			}
 
 			if (!current || !current->bli_materialized)
@@ -1383,14 +1413,14 @@ blb* BLB_open2(thread_db* tdbb,
 
 		vec<jrd_rel*>* vector = dbb->dbb_relations;
 
-		if (blob_id->bid_internal.bid_relation_id >= vector->count() ||
-			!(blob->blb_relation = (*vector)[blob_id->bid_internal.bid_relation_id] ) )
+		if (blobId.bid_internal.bid_relation_id >= vector->count() ||
+			!(blob->blb_relation = (*vector)[blobId.bid_internal.bid_relation_id] ) )
 		{
 				ERR_post(Arg::Gds(isc_bad_segstr_id));
 		}
 
 		blob->blb_pg_space_id = blob->blb_relation->getPages(tdbb)->rel_pg_space_id;
-		DPM_get_blob(tdbb, blob, blob_id->get_permanent_number(), false, (SLONG) 0);
+		DPM_get_blob(tdbb, blob, blobId.get_permanent_number(), false, (SLONG) 0);
 
 		// If the blob is known to be damaged, ignore it.
 
