@@ -318,6 +318,8 @@ void IDX_create_index(thread_db* tdbb,
 		SORT_init(dbb, &transaction->tra_sorts, key_length + sizeof(index_sort_record),
 				  2, 1, key_desc, callback, callback_arg);
 
+	IndexErrorContext context(relation, idx, index_name);
+
 	try {
 
 	jrd_rel* partner_relation = 0;
@@ -348,8 +350,6 @@ void IDX_create_index(thread_db* tdbb,
 		}
 	}
 
-	IndexErrorContext context(relation, idx, index_name);
-
 	// Loop thru the relation computing index keys.  If there are old versions, find them, too.
 	temporary_key key;
 	while (DPM_next(tdbb, &primary, LCK_read,
@@ -532,7 +532,6 @@ void IDX_create_index(thread_db* tdbb,
 				VIO_data(tdbb, &primary, dbb->dbb_permanent);
 				error_record = primary.rpb_record;
 			}
-
 		}
 
 		context.raise(tdbb, idx_e_duplicate, error_record);
@@ -548,6 +547,27 @@ void IDX_create_index(thread_db* tdbb,
 
 	BTR_create(tdbb, relation, idx, key_length, sort_handle, selectivity);
 
+	if (ifl_data.ifl_duplicates > 0)
+	{
+		AutoPtr<Record> error_record;
+		primary.rpb_record = NULL;
+		fb_assert(ifl_data.ifl_dup_recno >= 0);
+		primary.rpb_number.setValue(ifl_data.ifl_dup_recno);
+
+		if (DPM_get(tdbb, &primary, LCK_read))
+		{
+			if (primary.rpb_flags & rpb_deleted)
+				CCH_RELEASE(tdbb, &primary.getWindow(tdbb));
+			else
+			{
+				VIO_data(tdbb, &primary, dbb->dbb_permanent);
+				error_record = primary.rpb_record;
+			}
+		}
+
+		context.raise(tdbb, idx_e_duplicate, error_record);
+	}
+
 	if ((relation->rel_flags & REL_temp_conn) &&
 		(relation->getPages(tdbb)->rel_instance_id != 0))
 	{
@@ -771,6 +791,9 @@ void IDX_garbage_collect(thread_db*			tdbb,
 				idx_e result = BTR_key(tdbb, rpb->rpb_relation, rec1, &idx, &key1, 0, false);
 				if (result != idx_e_ok)
 				{
+					if (result == idx_e_conversion)
+						continue;
+
 					CCH_RELEASE(tdbb, &window);
 					context.raise(tdbb, result, rec1);
 				}
@@ -785,6 +808,9 @@ void IDX_garbage_collect(thread_db*			tdbb,
 					result = BTR_key(tdbb, rpb->rpb_relation, rec2, &idx, &key2, 0, false);
 					if (result != idx_e_ok)
 					{
+						if (result == idx_e_conversion)
+							continue;
+
 						CCH_RELEASE(tdbb, &window);
 						context.raise(tdbb, result, rec2);
 					}
@@ -805,6 +831,9 @@ void IDX_garbage_collect(thread_db*			tdbb,
 					result = BTR_key(tdbb, rpb->rpb_relation, rec3, &idx, &key2, 0, false);
 					if (result != idx_e_ok)
 					{
+						if (result == idx_e_conversion)
+							continue;
+
 						CCH_RELEASE(tdbb, &window);
 						context.raise(tdbb, result, rec3);
 					}
@@ -820,9 +849,7 @@ void IDX_garbage_collect(thread_db*			tdbb,
 				BTR_remove(tdbb, &window, &insertion);
 				root = (index_root_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_root);
 				if (stack1.hasMore(1))
-				{
 					BTR_description(tdbb, rpb->rpb_relation, root, &idx, i);
-				}
 			}
 		}
 	}
