@@ -108,10 +108,6 @@ static int process_id;
 //#define SHMEM_DELTA	(1 << 22)
 //#endif
 
-//#ifndef SIGURG
-//#define SIGURG		SIGINT
-//#endif
-
 #ifndef HAVE_SEMUN
 union semun
 {
@@ -365,8 +361,6 @@ private:
 					rwlocks->remove(getLockId());
 					delete rwcl;
 				}
-#else /* USE_FCNTL */
-				unlock();
 #endif /* USE_FCNTL */
 			}
 			else
@@ -1862,11 +1856,7 @@ int ISC_event_wait(event_t* event,
 		{
 			ret = pthread_cond_timedwait(event->event_cond, event->event_mutex, &timer);
 
-#if (defined LINUX || defined DARWIN || defined HP11 || defined FREEBSD)
 			if (ret == ETIMEDOUT)
-#else
-			if (ret == ETIME)
-#endif
 			{
 
 				/* The timer expired - see if the event occurred and return
@@ -2094,7 +2084,7 @@ void ISC_exception_post(ULONG sig_num, const TEXT* err_msg)
 		break;
 	default:
 		sprintf(log_msg, "%s Unknown Exception.\n"
-				"\t\tException number %"ULONGFORMAT"."
+				"\t\tException number %" ULONGFORMAT"."
 				"\tThis exception will cause the Firebird server\n"
 				"\tto terminate abnormally.", err_msg, sig_num);
 		break;
@@ -2257,7 +2247,7 @@ ULONG ISC_exception_post(ULONG except_code, const TEXT* err_msg)
 		break;
 	default:
 		sprintf (log_msg, "%s An exception occurred that does\n"
-				"\t\tnot have a description.  Exception number %"XLONGFORMAT".\n"
+				"\t\tnot have a description.  Exception number %" XLONGFORMAT".\n"
 				"\tThis exception will cause the Firebird server\n"
 				"\tto terminate abnormally.", err_msg, except_code);
 		break;
@@ -2295,7 +2285,7 @@ void ISC_remove_map_file(const TEXT* filename)
 {
 #ifndef WIN_NT
 	TEXT expanded_filename[MAXPATHLEN];
-	gds__prefix_lock(expanded_filename, filename);
+	iscPrefixLock(expanded_filename, filename, false);
 
 	// We can't do much (specially in dtors) when it fails
 	// therefore do not check for errors - at least it's just /tmp.
@@ -2331,7 +2321,7 @@ UCHAR* ISC_map_file(ISC_STATUS* status_vector,
  **************************************/
 
 	TEXT expanded_filename[MAXPATHLEN];
-	gds__prefix_lock(expanded_filename, filename);
+	iscPrefixLock(expanded_filename, filename, true);
 
 /* make the complete filename for the init file this file is to be used as a
    master lock to eliminate possible race conditions with just a single file
@@ -2339,7 +2329,7 @@ UCHAR* ISC_map_file(ISC_STATUS* status_vector,
    lock to a LCK_SHARED lock is not atomic*/
 
 	TEXT init_filename[MAXPATHLEN];
-	gds__prefix_lock(init_filename, INIT_FILE);
+	iscPrefixLock(init_filename, INIT_FILE, true);
 
 	const bool trunc_flag = (length != 0);
 
@@ -2372,7 +2362,7 @@ UCHAR* ISC_map_file(ISC_STATUS* status_vector,
 	if (fdSem < 0)
 	{
 		TEXT sem_filename[MAXPATHLEN];
-		gds__prefix_lock(sem_filename, SEM_FILE);
+		iscPrefixLock(sem_filename, SEM_FILE, true);
 		const int f = openFile(sem_filename);
 		if (f == -1)
 		{
@@ -2553,7 +2543,7 @@ UCHAR* ISC_map_file(ISC_STATUS* status_vector,
 	int retry_count = 0;
 
 	TEXT expanded_filename[MAXPATHLEN];
-	gds__prefix_lock(expanded_filename, filename);
+	iscPrefixLock(expanded_filename, filename, true);
 
 	const bool trunc_flag = (length != 0);
 	bool init_flag = false;
@@ -2617,6 +2607,8 @@ UCHAR* ISC_map_file(ISC_STATUS* status_vector,
 			Arg::Gds(isc_unavailable).copyTo(status_vector);
 			return NULL;
 		}
+
+		SetHandleInformation(event_handle, HANDLE_FLAG_INHERIT, 0);
 	}
 
 	if (length == 0)
@@ -2741,6 +2733,7 @@ UCHAR* ISC_map_file(ISC_STATUS* status_vector,
 		CloseHandle(file_handle);
 		goto retry;
 	}
+	SetHandleInformation(header_obj, HANDLE_FLAG_INHERIT, 0);
 
 	ULONG* const header_address = (ULONG*) MapViewOfFile(header_obj, FILE_MAP_WRITE, 0, 0, 0);
 
@@ -2767,7 +2760,7 @@ UCHAR* ISC_map_file(ISC_STATUS* status_vector,
 /* Create the real file mapping object. */
 
 	TEXT mapping_name[64]; // enough for int32 as text
-	sprintf(mapping_name, "_mapping_%"ULONGFORMAT, header_address[1]);
+	sprintf(mapping_name, "_mapping_%" ULONGFORMAT, header_address[1]);
 
 	if (!make_object_name(object_name, sizeof(object_name), filename, mapping_name))
 	{
@@ -2793,6 +2786,7 @@ UCHAR* ISC_map_file(ISC_STATUS* status_vector,
 		CloseHandle(file_handle);
 		return NULL;
 	}
+	SetHandleInformation(file_obj, HANDLE_FLAG_INHERIT, 0);
 
 	UCHAR* const address = (UCHAR*) MapViewOfFile(file_obj, FILE_MAP_WRITE, 0, 0, 0);
 
@@ -3105,7 +3099,7 @@ int ISC_mutex_init(sh_mem* shmem_data, struct mtx* mutex, struct mtx** mapped)
 	SharedFile* sf = SharedFile::locate(mutex);
 	if (!sf)
 	{
-		gds__log("SharedFile::locate(sem) failed");
+		gds__log("SharedFile::locate(mutex) failed");
 
 #else // USE_FILELOCKS
 
@@ -3717,6 +3711,8 @@ static bool initializeFastMutex(FAST_MUTEX* lpMutex, LPSECURITY_ATTRIBUTES lpAtt
 
 	if (lpMutex->hEvent)
 	{
+		SetHandleInformation(lpMutex->hEvent, HANDLE_FLAG_INHERIT, 0);
+
 		if (lpName)
 			sprintf(sz, FAST_MUTEX_MAP_NAME, lpName);
 
@@ -3732,6 +3728,8 @@ static bool initializeFastMutex(FAST_MUTEX* lpMutex, LPSECURITY_ATTRIBUTES lpAtt
 
 		if (lpMutex->hFileMap)
 		{
+			SetHandleInformation(lpMutex->hFileMap, HANDLE_FLAG_INHERIT, 0);
+
 			lpMutex->lpSharedInfo = (FAST_MUTEX_SHARED_SECTION*)
 				MapViewOfFile(lpMutex->hFileMap, FILE_MAP_WRITE, 0, 0, 0);
 
@@ -4032,7 +4030,7 @@ UCHAR* ISC_remap_file(ISC_STATUS * status_vector,
 	while (true)
 	{
 		TEXT mapping_name[64]; // enough for int32 as text
-		sprintf(mapping_name, "_mapping_%"ULONGFORMAT, shmem_data->sh_mem_hdr_address[1] + 1);
+		sprintf(mapping_name, "_mapping_%" ULONGFORMAT, shmem_data->sh_mem_hdr_address[1] + 1);
 
 		TEXT object_name[MAXPATHLEN];
 		if (!make_object_name(object_name, sizeof(object_name), shmem_data->sh_mem_name, mapping_name))
@@ -4129,7 +4127,7 @@ void ISC_unmap_file(ISC_STATUS* status_vector, sh_mem* shmem_data)
  *	Unmap a given file.
  *
  **************************************/
-#ifdef USE_SYS5SEMAPHORE
+#if defined(USE_SYS5SEMAPHORE)
 	// Lock init file.
 	FileLock initLock(fd_init, FileLock::OPENED);
 	if (!initLock.doLock(status_vector, FileLock::FLM_EXCLUSIVE))
@@ -4146,6 +4144,11 @@ void ISC_unmap_file(ISC_STATUS* status_vector, sh_mem* shmem_data)
 		SharedFile::remove(shmem_data->sh_mem_address);
 	}
 	--sharedCount;
+#elif defined(USE_FCNTL)
+	{
+		FileLock lock(shmem_data->sh_mem_handle, FileLock::OPENED);
+		lock.setLevel(FileLock::LCK_SHARED);
+	}
 #endif
 
 	munmap((char *) shmem_data->sh_mem_address, shmem_data->sh_mem_length_mapped);
@@ -4186,7 +4189,7 @@ void ISC_unmap_file(ISC_STATUS* status_vector, sh_mem* shmem_data)
 	CloseHandle(shmem_data->sh_mem_hdr_object);
 
 	TEXT expanded_filename[MAXPATHLEN];
-	gds__prefix_lock(expanded_filename, shmem_data->sh_mem_name);
+	iscPrefixLock(expanded_filename, shmem_data->sh_mem_name, false);
 
 	// Delete file only if it is not used by anyone else
 	HANDLE hFile = CreateFile(expanded_filename,
