@@ -64,6 +64,9 @@
 #define CORRUPT(number)         ERR_corrupt (number)
 #define IBERROR(number)         ERR_error (number)
 
+#ifdef WIN_NT
+#define REEXPAND_DBNAME
+#endif
 
 #define BLKCHK(blk, type)       if (!blk->checkHandle()) BUGCHECK(147)
 
@@ -105,7 +108,7 @@ namespace Jrd {
 
 const int QUANTUM			= 100;	// Default quantum
 const int SWEEP_QUANTUM		= 10;	// Make sweeps less disruptive
-const int MAX_CALLBACKS		= 50;
+const ULONG MAX_CALLBACKS		= 50;
 
 // fwd. decl.
 class thread_db;
@@ -144,7 +147,7 @@ class Trigger
 {
 public:
 	Firebird::HalfStaticArray<UCHAR, 128> blr;	// BLR code
-	bid			dbg_blob_id;					// RDB$DEBUG_INFO
+	Firebird::HalfStaticArray<UCHAR, 128> debugInfo;	// debug info
 	jrd_req*	request;					// Compiled request. Gets filled on first invocation
 	bool		compile_in_progress;
 	bool		sys_trigger;
@@ -156,10 +159,8 @@ public:
 	void release(thread_db*);				// Try to free trigger request
 
 	explicit Trigger(MemoryPool& p)
-		: blr(p), name(p)
-	{
-		dbg_blob_id.clear();
-	}
+		: blr(p), debugInfo(p), name(p)
+	{}
 };
 
 
@@ -187,6 +188,7 @@ const int DYN_REQUESTS				= 2;
 //
 // Errors during validation - will be returned on info calls
 // CVC: It seems they will be better in a header for val.cpp that's not val.h
+// Do not forget to add new constants to inf.cpp:INF_database_info
 //
 const int VAL_PAG_WRONG_TYPE			= 0;
 const int VAL_PAG_CHECKSUM_ERR			= 1;
@@ -214,7 +216,13 @@ const int VAL_REL_CHAIN_ORPHANS			= 22;
 const int VAL_INDEX_MISSING_ROWS		= 23;
 const int VAL_INDEX_ORPHAN_CHILD		= 24;
 const int VAL_INDEX_CYCLE				= 25;
-const int VAL_MAX_ERROR					= 26;
+const int VAL_INDEX_BAD_LEFT_SIBLING	= 26;
+const int VAL_INDEX_MISSES_NODE			= 27;
+const int VAL_DATA_PAGE_ISNT_IN_PIP		= 28;
+const int VAL_DATA_PAGE_SLOT_NOT_FOUND	= 29;
+const int VAL_DATA_PAGE_SLOT_BAD_VAL	= 30;
+const int VAL_DATA_PAGE_HASNO_PP		= 31;
+const int VAL_MAX_ERROR					= 32;
 
 
 struct DSqlCacheItem
@@ -251,18 +259,7 @@ public:
 		}
 	}
 
-	static void destroy(Attachment* const attachment)
-	{
-		if (attachment)
-		{
-			Database* const dbb = attachment->att_database;
-			MemoryPool* const pool = attachment->att_pool;
-			Firebird::MemoryStats temp_stats;
-			pool->setStatsGroup(temp_stats);
-			delete attachment;
-			dbb->deletePool(pool);
-		}
-	}
+	static void destroy(Attachment* const attachment);
 
 /*	Attachment()
 	:	att_database(0),
@@ -304,6 +301,7 @@ public:
 	sort_context*	att_active_sorts;		// Active sorts
 	Lock*		att_id_lock;				// Attachment lock (if any)
 	SLONG		att_attachment_id;			// Attachment ID
+	Lock*		att_cancel_lock;			// lock to cancel the active request
 	const ULONG	att_lock_owner_id;			// ID for the lock manager
 	SLONG		att_lock_owner_handle;		// Handle for the lock manager
 	ULONG		att_backup_state_counter;	// Counter of backup state locks for attachment
@@ -333,7 +331,6 @@ public:
 	DSqlCache att_dsql_cache;	// DSQL cache locks
 	Firebird::SortedArray<void*> att_udf_pointers;
 	dsql_dbb* att_dsql_instance;
-	Firebird::Mutex att_mutex;				// attachment mutex
 
 	EDS::Connection* att_ext_connection;	// external connection executed by this attachment
 	ULONG att_ext_call_depth;				// external connection call depth, 0 for user attachment
@@ -346,23 +343,14 @@ public:
 	PreparedStatement* prepareStatement(thread_db* tdbb, Firebird::MemoryPool& pool,
 		jrd_tra* transaction, const Firebird::string& text);
 
-	void cancelExternalConnection(thread_db* tdbb);
+	void signalCancel(thread_db* tdbb);
+	void signalShutdown(thread_db* tdbb);
 
 	bool backupStateWriteLock(thread_db* tdbb, SSHORT wait);
 	void backupStateWriteUnLock(thread_db* tdbb);
 	bool backupStateReadLock(thread_db* tdbb, SSHORT wait);
 	void backupStateReadUnLock(thread_db* tdbb);
 
-	bool checkHandle() const
-	{
-		if (!isKnownHandle())
-		{
-			return false;
-		}
-
-		return TypedHandle<type_att>::checkHandle();
-	}
-
 private:
 	Attachment(MemoryPool* pool, Database* dbb);
 	~Attachment();
@@ -371,31 +359,32 @@ private:
 
 // Attachment flags
 
-const ULONG ATT_no_cleanup			= 1;	// Don't expunge, purge, or garbage collect
-const ULONG ATT_shutdown			= 2;	// attachment has been shutdown
-const ULONG ATT_purge_error			= 4;	// trouble happened in purge attachment, att_mutex remains locked
-const ULONG ATT_shutdown_manager	= 8;	// attachment requesting shutdown
-const ULONG ATT_lck_init_done		= 16;	// LCK_init() called for the attachment
-const ULONG ATT_exclusive			= 32;	// attachment wants exclusive database access
-const ULONG ATT_attach_pending		= 64;	// Indicate attachment is only pending
-const ULONG ATT_exclusive_pending	= 128;	// Indicate exclusive attachment pending
-const ULONG ATT_gbak_attachment		= 256;	// Indicate GBAK attachment
+const ULONG ATT_no_cleanup			= 0x1;		// Don't expunge, purge, or garbage collect
+const ULONG ATT_shutdown			= 0x2;		// attachment has been shutdown
+//const ULONG ATT_purge_error			= 0x4;		// trouble happened in purge attachment
+const ULONG ATT_shutdown_manager	= 0x8;		// attachment requesting shutdown
+const ULONG ATT_lck_init_done		= 0x10;		// LCK_init() called for the attachment
+const ULONG ATT_exclusive			= 0x20;		// attachment wants exclusive database access
+const ULONG ATT_attach_pending		= 0x40;		// Indicate attachment is only pending
+const ULONG ATT_exclusive_pending	= 0x80;		// Indicate exclusive attachment pending
+const ULONG ATT_gbak_attachment		= 0x100;	// Indicate GBAK attachment
 
 #ifdef GARBAGE_THREAD
-const ULONG ATT_notify_gc			= 1024;	// Notify garbage collector to expunge, purge ..
-const ULONG ATT_disable_notify_gc	= 2048;	// Temporarily perform own garbage collection
-const ULONG ATT_garbage_collector	= 4096;	// I'm a garbage collector
+const ULONG ATT_notify_gc			= 0x200;	// Notify garbage collector to expunge, purge ..
+const ULONG ATT_disable_notify_gc	= 0x400;	// Temporarily perform own garbage collection
+const ULONG ATT_garbage_collector	= 0x800;	// I'm a garbage collector
 
 const ULONG ATT_NO_CLEANUP			= (ATT_no_cleanup | ATT_notify_gc);
 #else
 const ULONG ATT_NO_CLEANUP			= ATT_no_cleanup;
 #endif
 
-const ULONG ATT_cancel_raise		= 8192;		// Cancel currently running operation
-const ULONG ATT_cancel_disable		= 16384;	// Disable cancel operations
-const ULONG ATT_gfix_attachment		= 32768;	// Indicate a GFIX attachment
-const ULONG ATT_gstat_attachment	= 65536;	// Indicate a GSTAT attachment
-const ULONG ATT_no_db_triggers		= 131072;	// Don't execute database triggers
+const ULONG ATT_cancel_raise		= 0x1000;	// Cancel currently running operation
+const ULONG ATT_cancel_disable		= 0x2000;	// Disable cancel operations
+const ULONG ATT_gfix_attachment		= 0x4000;	// Indicate a GFIX attachment
+const ULONG ATT_gstat_attachment	= 0x8000;	// Indicate a GSTAT attachment
+const ULONG ATT_no_db_triggers		= 0x10000;	// Don't execute database triggers
+const ULONG ATT_manual_lock			= 0x20000;	// Was locked manually
 
 
 inline bool Attachment::locksmith() const
@@ -404,7 +393,6 @@ inline bool Attachment::locksmith() const
 }
 
 
-
 // Procedure block
 
 class jrd_prc : public pool_alloc<type_prc>
@@ -800,7 +788,8 @@ const USHORT TDBB_verb_cleanup			= 256;	// verb cleanup is in progress
 const USHORT TDBB_use_db_page_space		= 512;	// use database (not temporary) page space in GTT operations
 const USHORT TDBB_detaching				= 1024;	// detach is in progress
 const USHORT TDBB_wait_cancel_disable	= 2048;	// don't cancel current waiting operation
-
+const USHORT TDBB_cache_unwound			= 4096;	// page cache was unwound 
+const USHORT TDBB_dfw_cleanup			= 8192;	// DFW cleanup phase is active
 
 class ThreadContextHolder
 {
@@ -876,6 +865,53 @@ private:
 };
 
 
+// AST attachment holder - controls AST mutex in attachment block
+
+class AstAttachmentHolder
+{
+public:
+	explicit AstAttachmentHolder(Attachment* attachment);
+
+	~AstAttachmentHolder()
+	{
+		if (mtx)
+		{
+			destroy();
+		}
+	}
+
+private:
+	Firebird::ExistenceMutex* mtx;
+
+	void destroy();
+};
+
+
+// Checks that passed pointer is not NULL
+
+class AttachmentNotNull
+{
+public:
+	explicit AttachmentNotNull(Attachment* attachment);
+	AttachmentNotNull();
+};
+
+
+
+// AST routines context helper
+
+class AstContextHolder :
+	public AttachmentNotNull,		// when needed checks NULL pointer passed
+	public ThreadContextHolder,		// creates thread_db block for AST routine
+	public AstAttachmentHolder,		// controls AST mutex in attachment block
+	public Database::SyncGuard		// controls dbb_sync mutex
+{
+public:
+	explicit AstContextHolder(Database* dbb, Attachment* attachment = NULL);
+	AstContextHolder(ISC_STATUS* status, Attachment* attachment);
+};
+
+
 // duplicate context of firebird string to store in jrd_nod::nod_arg
 inline char* stringDup(MemoryPool& p, const Firebird::string& s)
 {
@@ -1019,4 +1055,8 @@ namespace Jrd {
 	typedef Firebird::SubsystemContextPoolHolder <Jrd::thread_db, MemoryPool> ContextPoolHolder;
 }
 
+#ifdef REEXPAND_DBNAME
+Firebird::Mutex& JRD_get_dbinitmutex();
+#endif
+
 #endif // JRD_JRD_H
