@@ -231,21 +231,25 @@ public:
 			m_save_lock = att->att_wait_lock;
 
 		m_cancel_disabled = (m_tdbb->tdbb_flags & TDBB_wait_cancel_disable); 
-		m_tdbb->tdbb_flags |= TDBB_wait_cancel_disable;
+		if (wait == LCK_WAIT)
+		{
+			switch (lock->lck_type)
+			{
+			case LCK_tra:
+				m_tdbb->tdbb_flags &= ~TDBB_wait_cancel_disable;
+				if (att)
+					att->att_wait_lock = lock;
+			break;
 
-		if (!wait)
-			return;
-
-		switch (lock->lck_type)
+			default:
+				m_tdbb->tdbb_flags |= TDBB_wait_cancel_disable;
+			}
+		}
+		else
 		{
-		case LCK_tra:
 			m_tdbb->tdbb_flags &= ~TDBB_wait_cancel_disable;
 			if (att)
 				att->att_wait_lock = lock;
-		break;
-
-		default:
-			;
 		}
 	}
 
@@ -487,6 +491,7 @@ SLONG LCK_get_owner_handle(thread_db* tdbb, enum lck_t lock_type)
 	case LCK_bdb:
 	case LCK_rel_exist:
 	case LCK_rel_partners:
+	case LCK_rel_rescan:
 	case LCK_idx_exist:
 	case LCK_shadow:
 	case LCK_retaining:
@@ -498,6 +503,7 @@ SLONG LCK_get_owner_handle(thread_db* tdbb, enum lck_t lock_type)
 	case LCK_tt_exist:
 	case LCK_shared_counter:
 	case LCK_sweep:
+	case LCK_rel_gc:
 		handle = *LCK_OWNER_HANDLE_DBB(tdbb);
 		break;
 	case LCK_attachment:
@@ -918,7 +924,7 @@ static void enqueue(thread_db* tdbb, Lock* lock, USHORT level, SSHORT wait)
 }
 
 
-static int external_ast(void* lock_void)
+static int external_ast(void* ast_object)
 {
 /**************************************
  *
@@ -932,21 +938,29 @@ static int external_ast(void* lock_void)
  *	we are blocking a lock from another process.
  *
  **************************************/
-	Lock* lock = static_cast<Lock*>(lock_void);
+	Lock* const lock = static_cast<Lock*>(ast_object);
 	fb_assert(LCK_CHECK_LOCK(lock));
 
-	// go through the list, saving the next lock in the list
-	// in case the current one gets deleted in the ast
-
-	Lock* next;
-	for (Lock* match = hash_get_lock(lock, 0, 0); match; match = next)
+	try
 	{
-		next = match->lck_identical;
-		if (match->lck_ast) {
-			(*match->lck_ast)(match->lck_object);
+		AstContextHolder tdbb(lock->lck_dbb, lock->lck_attachment);
+
+		// go through the list, saving the next lock in the list
+		// in case the current one gets deleted in the ast
+
+		Lock* next;
+		for (Lock* match = hash_get_lock(lock, 0, 0); match; match = next)
+		{
+			next = match->lck_identical;
+			if (match->lck_ast) {
+				(*match->lck_ast)(match->lck_object);
+			}
 		}
 	}
-	return 0; // make the compiler happy
+	catch (const Firebird::Exception&)
+	{} // no-op
+
+	return 0;
 }
 
 
