@@ -56,7 +56,9 @@ enum lck_t {
 	LCK_cancel,					// Cancellation lock
 	LCK_btr_dont_gc,			// Prevent removal of b-tree page from index
 	LCK_shared_counter,			// Database-wide shared counter
-	LCK_tra_pc					// Precommitted transaction lock
+	LCK_tra_pc,					// Precommitted transaction lock
+	LCK_rel_gc,					// Allow garbage collection for relation
+	LCK_rel_rescan				// Relation forced rescan lock
 };
 
 // Lock owner types
