@@ -62,7 +62,7 @@ static void expand_index(WIN *);
 static btree_exp* find_current(exp_index_buf*, Ods::btree_page*, const UCHAR*);
 static bool find_saved_node(thread_db* tdbb, RecordSource*, IRSB_NAV, WIN*, UCHAR**);
 static UCHAR* get_position(thread_db*, RecordSource*, IRSB_NAV, WIN *, rse_get_mode, btree_exp**);
-static bool get_record(thread_db* tdbb, RecordSource*, IRSB_NAV, record_param*, temporary_key*, bool);
+static bool get_record(thread_db* tdbb, RecordSource*, IRSB_NAV, record_param*, temporary_key*);
 static void init_fetch(IRSB_NAV);
 static UCHAR* nav_open(thread_db* tdbb, RecordSource*, IRSB_NAV, WIN *, rse_get_mode); //, btree_exp**);
 static void set_page(IRSB_NAV impure, WIN* window);
@@ -209,8 +209,10 @@ bool NAV_get_record(thread_db* tdbb, RecordSource* rsb,
 		return false;
 	temporary_key key;
 	memcpy(key.key_data, impure->irsb_nav_data, impure->irsb_nav_length);
-	jrd_nod* retrieval_node = (jrd_nod*) rsb->rsb_arg[RSB_NAV_index];
-	IndexRetrieval* retrieval = (IndexRetrieval*) retrieval_node->nod_arg[e_idx_retrieval];
+
+	const jrd_nod* const retrieval_node = (jrd_nod*) rsb->rsb_arg[RSB_NAV_index];
+	const IndexRetrieval* const retrieval = (IndexRetrieval*) retrieval_node->nod_arg[e_idx_retrieval];
+	const USHORT compare_flags = retrieval->irb_generic & (irb_descending | irb_partial | irb_starting);
 
 	// set the upper (or lower) limit for navigational retrieval
 	temporary_key upper;
@@ -369,16 +371,14 @@ bool NAV_get_record(thread_db* tdbb, RecordSource* rsb,
 
 		// Make sure we haven't hit the upper (or lower) limit.
 		if ((direction == RSE_get_forward) && retrieval->irb_upper_count &&
-			compare_keys(idx, key.key_data, key.key_length, &upper,
-						 retrieval->irb_generic & (irb_descending | irb_partial | irb_starting)) > 0)
+			compare_keys(idx, key.key_data, key.key_length, &upper, compare_flags) > 0)
 		{
 			break;
 		}
 
 #ifdef SCROLLABLE_CURSORS
 		if ((direction == RSE_get_backward) && retrieval->irb_lower_count &&
-			compare_keys(idx, key.key_data, key.key_length, &lower,
-						 retrieval->irb_generic & (irb_descending | irb_partial | irb_starting)) < 0)
+			compare_keys(idx, key.key_data, key.key_length, &lower, compare_flags) < 0)
 		{
 			break;
 		}
@@ -418,7 +418,7 @@ bool NAV_get_record(thread_db* tdbb, RecordSource* rsb,
 
 		CCH_RELEASE(tdbb, &window);
 
-		if (get_record(tdbb, rsb, impure, rpb, &key, false)) {
+		if (get_record(tdbb, rsb, impure, rpb, &key)) {
 			return true;
 		}
 
@@ -680,6 +680,10 @@ static bool find_saved_node(thread_db* tdbb, RecordSource* rsb, IRSB_NAV impure,
 	index_desc* idx = (index_desc*) ((SCHAR*) impure + (IPTR) rsb->rsb_arg[RSB_NAV_idx_offset]);
 	Ods::btree_page* page = (Ods::btree_page*) CCH_FETCH(tdbb, window, LCK_read, pag_index);
 
+	const jrd_nod* const retrieval_node = (jrd_nod*) rsb->rsb_arg[RSB_NAV_index];
+	const IndexRetrieval* const retrieval = (IndexRetrieval*) retrieval_node->nod_arg[e_idx_retrieval];
+	const USHORT compare_flags = retrieval->irb_generic & (irb_descending | irb_partial | irb_starting);
+
 	// the outer loop goes through all the sibling pages
 	// looking for the node (in case the page has split);
 	// the inner loop goes through the nodes on each page
@@ -711,7 +715,7 @@ static bool find_saved_node(thread_db* tdbb, RecordSource* rsb, IRSB_NAV impure,
 			memcpy(key.key_data + node.prefix, node.data, node.length);
 			key.key_length = node.length + node.prefix;
 			const int result = compare_keys(idx, impure->irsb_nav_data,
-											impure->irsb_nav_length, &key, 0);
+											impure->irsb_nav_length, &key, compare_flags);
 
 			// if the keys are equal, return this node even if it is just a duplicate node;
 			// duplicate nodes that have already been returned will be filtered out at a
@@ -876,7 +880,7 @@ static UCHAR* get_position(thread_db* tdbb,
 
 
 static bool get_record(thread_db* tdbb, RecordSource* rsb, IRSB_NAV impure,
-						record_param* rpb, temporary_key* key, bool inhibit_cleanup)
+					   record_param* rpb, temporary_key* key)
 {
 /**************************************
  *
@@ -892,66 +896,39 @@ static bool get_record(thread_db* tdbb, RecordSource* rsb, IRSB_NAV impure,
 	jrd_req* request = tdbb->getRequest();
 	index_desc* idx = (index_desc*) ((SCHAR*) impure + (IPTR) rsb->rsb_arg[RSB_NAV_idx_offset]);
 
-	ULONG old_att_flags = 0;
-	bool result = false;
-
-	try {
-
-		if (inhibit_cleanup)
-		{
-			// Inhibit garbage collection & other housekeeping -
-			// to prevent a deadlock when we visit the record.
-			// There are cases when we are called here and have a window locked
-			// HACK for bug 7041
-
-			old_att_flags = (tdbb->getAttachment()->att_flags & ATT_no_cleanup);
-			tdbb->getAttachment()->att_flags |= ATT_no_cleanup;	// HACK
-		}
-
 #ifdef SCROLLABLE_CURSORS
-		// the attempt to get a record, whether successful or not, takes
-		// us off bof or eof (otherwise we will keep trying to retrieve
-		// the first record)
+	// the attempt to get a record, whether successful or not, takes
+	// us off bof or eof (otherwise we will keep trying to retrieve
+	// the first record)
 
-		impure->irsb_flags &= ~(irsb_bof | irsb_eof);
+	impure->irsb_flags &= ~(irsb_bof | irsb_eof);
 #endif
 
-		result = VIO_get(tdbb, rpb, request->req_transaction, request->req_pool);
+	if (!VIO_get(tdbb, rpb, request->req_transaction, request->req_pool))
+		return false;
 
-		temporary_key value;
-		if (result)
-		{
-			BTR_key(tdbb, rpb->rpb_relation, rpb->rpb_record,
-					reinterpret_cast<index_desc*>((SCHAR*) impure +
-						(IPTR) rsb->rsb_arg[RSB_NAV_idx_offset]),
-					&value,	0, false);
-			if (compare_keys(idx, key->key_data, key->key_length, &value, 0)) {
-				result = false;
-			}
-			else
-			{
-				// mark in the navigational bitmap that we have visited this record
-				RBM_SET(tdbb->getDefaultPool(), &impure->irsb_nav_records_visited,
-					rpb->rpb_number.getValue());
-			}
-		}
+	temporary_key value;
 
-		if (inhibit_cleanup)
-		{
-			tdbb->getAttachment()->att_flags &= ~ATT_no_cleanup;
-			tdbb->getAttachment()->att_flags |= old_att_flags;
-		}
+	const idx_e result =
+		BTR_key(tdbb, rpb->rpb_relation, rpb->rpb_record,
+				reinterpret_cast<index_desc*>((SCHAR*) impure +
+					(IPTR) rsb->rsb_arg[RSB_NAV_idx_offset]),
+				&value, 0, false);
 
-	}	// try
-	catch (const Firebird::Exception&)
+	if (result != idx_e_ok)
 	{
-		// Cleanup the HACK should there be an error
-		tdbb->getAttachment()->att_flags &= ~ATT_no_cleanup;
-		tdbb->getAttachment()->att_flags |= old_att_flags;
-		throw;
+		IndexErrorContext context(rpb->rpb_relation, idx);
+		context.raise(tdbb, result, rpb->rpb_record);
 	}
 
-	return result;
+	if (compare_keys(idx, key->key_data, key->key_length, &value, 0))
+		return false;
+
+	// mark in the navigational bitmap that we have visited this record
+	RBM_SET(tdbb->getDefaultPool(), &impure->irsb_nav_records_visited,
+		rpb->rpb_number.getValue());
+
+	return true;
 }
 
 
@@ -1099,8 +1076,9 @@ static UCHAR* nav_open(thread_db* tdbb,
 	{
 
 		// If END_BUCKET is reached BTR_find_leaf will return NULL
-		while (!(pointer = BTR_find_leaf(page, limit_ptr, impure->irsb_nav_data,
-									  0, (idx->idx_flags & idx_descending) != 0, true)))
+		while (!(pointer = BTR_find_leaf(page, limit_ptr, impure->irsb_nav_data, NULL,
+							(idx->idx_flags & idx_descending) != 0, 
+							(retrieval->irb_generic & (irb_starting | irb_partial)) )))
 		{
 			page = (Ods::btree_page*) CCH_HANDOFF(tdbb, window, page->btr_sibling,
 				  LCK_read, pag_index);
@@ -1260,4 +1238,3 @@ static bool setup_bitmaps(thread_db* tdbb, RecordSource* rsb, IRSB_NAV impure)
 
 	return true;
 }
-
