@@ -164,9 +164,11 @@ static const int CLASS_LINUX_SHEB = 39;		// LINUX/SH (big-endian)
 static const int CLASS_LINUX_HPPA = 40;		// LINUX/HPPA
 static const int CLASS_LINUX_ALPHA = 41;	// LINUX/ALPHA
 static const int CLASS_LINUX_ARM64 = 42;	// LINUX/ARM64
+static const int CLASS_LINUX_PPC64EL = 43;	// LINUX/PowerPc64EL
+
 
 static const int CLASS_MAX10 = CLASS_LINUX_AMD64;	// This should not be changed, no new ports with ODS10
-static const int CLASS_MAX = CLASS_LINUX_ARM64;
+static const int CLASS_MAX = CLASS_LINUX_PPC64EL;
 
 // ARCHITECTURE COMPATIBILITY CLASSES
 
@@ -266,7 +268,9 @@ static const ArchitectureType archMatrix[CLASS_MAX + 1] =
 	archBigEndian,    // CLASS_LINUX_SHEB
 	archBigEndian,    // CLASS_LINUX_HPPA
 	archLittleEndian, // CLASS_LINUX_ALPHA
-	archLittleEndian  // CLASS_LINUX_ARM64
+	archLittleEndian, // CLASS_LINUX_ARM64
+	archLittleEndian  // CLASS_LINUX_PPC64EL
+
 };
 
 #ifdef __sun
@@ -330,8 +334,10 @@ const SSHORT CLASS		= CLASS_LINUX_SHEB;
 const SSHORT CLASS		= CLASS_LINUX_HPPA;
 #elif defined(ALPHA)
 const SSHORT CLASS		= CLASS_LINUX_ALPHA;
-//#elif defined(ARM64)
-//const SSHORT CLASS		= CLASS_LINUX_ARM64;
+#elif defined(ARM64)
+const SSHORT CLASS		= CLASS_LINUX_ARM64;
+#elif defined(PPC64EL)
+const SSHORT CLASS		= CLASS_LINUX_PPC64EL;
 #else
 #error no support on other hardware for Linux
 #endif
@@ -954,7 +960,7 @@ PAG PAG_allocate(thread_db* tdbb, WIN* window)
 		CCH_precedence(tdbb, window, pip_window.win_page);
 #ifdef VIO_DEBUG
 		if (debug_flag > DEBUG_WRITES_INFO)
-			printf("\tPAG_allocate:  allocated page %"SLONGFORMAT"\n",
+			printf("\tPAG_allocate:  allocated page %" SLONGFORMAT"\n",
 					  window->win_page.getPageNum());
 #endif
 		return new_page;
@@ -1791,7 +1797,7 @@ void PAG_release_page(thread_db* tdbb, const PageNumber& number, const PageNumbe
 
 #ifdef VIO_DEBUG
 	if (debug_flag > DEBUG_WRITES_INFO)
-		printf("\tPAG_release_page:  about to release page %"SLONGFORMAT"\n", number.getPageNum());
+		printf("\tPAG_release_page:  about to release page %" SLONGFORMAT"\n", number.getPageNum());
 #endif
 
 	PageManager& pageMgr = dbb->dbb_page_manager;
@@ -2365,11 +2371,14 @@ ULONG PageSpace::maxAlloc(const USHORT pageSize)
  *
  **************************************/
 	const jrd_file* f = file;
-	while (f->fil_next) {
+	ULONG nPages = PIO_get_number_of_pages(f, pageSize);
+	while (f->fil_next && (f->fil_max_page - f->fil_min_page + 1 + f->fil_fudge) == nPages) 
+	{
 		f = f->fil_next;
+		nPages = PIO_get_number_of_pages(f, pageSize);
 	}
 
-	const ULONG nPages = f->fil_min_page - f->fil_fudge + PIO_get_number_of_pages(f, pageSize);
+	nPages += f->fil_min_page - f->fil_fudge;
 
 	if (maxPageNumber < nPages)
 		maxPageNumber = nPages;
