@@ -71,6 +71,7 @@
 #include "../jrd/trace/TraceManager.h"
 #include "../jrd/trace/TraceObjects.h"
 #include "../jrd/trace/TraceService.h"
+#include "../jrd/val_proto.h"
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -273,26 +274,6 @@ namespace {
 	typedef Array<Jrd::Service*> AllServices;
 	GlobalPtr<AllServices> allServices;	// protected by globalServicesMutex
 	volatile bool svcShutdown = false;
-
-	void put_status_arg(ISC_STATUS*& status, const MsgFormat::safe_cell& value)
-	{
-		using MsgFormat::safe_cell;
-
-		switch (value.type)
-		{
-		case safe_cell::at_int64:
-		case safe_cell::at_uint64:
-			*status++ = isc_arg_number;
-			*status++ = static_cast<SLONG>(value.i_value); // May truncate number!
-			break;
-		case safe_cell::at_str:
-			*status++ = isc_arg_string;
-			*status++ = (ISC_STATUS) (IPTR) value.st_value.s_string;
-			break;
-		default:
-			break;
-		}
-	}
 } // anonymous namespace
 
 
@@ -512,6 +493,7 @@ void Service::setServiceStatus(const USHORT facility, const USHORT errcode, cons
 
 	// We preserve the five params of the old code.
 	// Don't want to overflow the status vector.
+	svc_arg_ptr = svc_arg_conv;
 	for (unsigned int loop = 0; loop < 5 && loop < args.getCount(); ++loop)
 	{
 		put_status_arg(status, args.getCell(loop));
@@ -590,6 +572,36 @@ void Service::setServiceStatus(const USHORT facility, const USHORT errcode, cons
 	makePermanentStatusVector();
 }
 
+void Service::put_status_arg(ISC_STATUS*& status, const MsgFormat::safe_cell& value)
+{
+	using MsgFormat::safe_cell;
+
+	switch (value.type)
+	{
+	case safe_cell::at_int64:
+	case safe_cell::at_uint64:
+		*status++ = isc_arg_number;
+		*status++ = static_cast<SLONG>(value.i_value); // May truncate number!
+		break;
+	case safe_cell::at_str:
+		*status++ = isc_arg_string;
+		*status++ = (ISC_STATUS) (IPTR) value.st_value.s_string;
+		break;
+	case safe_cell::at_char:
+		svc_arg_ptr[0] = value.c_value;
+		svc_arg_ptr[1] = 0;
+
+		*status++ = isc_arg_string;
+		*status++ = (ISC_STATUS) (IPTR) &svc_arg_ptr[0];
+
+		svc_arg_ptr += 2;
+		break;
+	default:
+		break;
+	}
+}
+
+
 void Service::hidePasswd(ArgvType&, int)
 {
 	// no action
@@ -657,12 +669,14 @@ THREAD_ENTRY_DECLARE main_gstat(THREAD_ENTRY_PARAM arg);
 #define MAIN_GSTAT		main_gstat
 #define MAIN_NBAK		NBACKUP_main
 #define MAIN_TRACE		TRACE_main
+#define MAIN_VALIDATE	VAL_service
 #else
 #define MAIN_GBAK		NULL
 #define MAIN_GFIX		NULL
 #define MAIN_GSTAT		NULL
 #define MAIN_NBAK		NULL
 #define MAIN_TRACE		NULL
+#define MAIN_VALIDATE	NULL
 #endif
 
 #if !defined(EMBEDDED) && !defined(BOOT_BUILD)
@@ -726,6 +740,7 @@ static const serv_entry services[] =
 	{ isc_action_svc_set_mapping, "Set Domain Admins Mapping to RDB$ADMIN", NULL, MAIN_GSEC },
 	{ isc_action_svc_drop_mapping, "Drop Domain Admins Mapping to RDB$ADMIN", NULL, MAIN_GSEC },
 	{ isc_action_svc_display_user_adm, "Display User with Admin Info", NULL, MAIN_GSEC },
+	{ isc_action_svc_validate, "Validate Database", NULL, MAIN_VALIDATE },
 /* actions with no names are undocumented */
 	{ isc_action_svc_set_config, NULL, NULL, TEST_THREAD },
 	{ isc_action_svc_default_config, NULL, NULL, TEST_THREAD },
@@ -802,8 +817,14 @@ Service::Service(const TEXT* service_name, USHORT spb_length, const UCHAR* spb_d
 		ClumpletReader spb(ClumpletReader::SpbAttach, spb_data, spb_length);
 		Options options(spb);
 
-		// Perhaps checkout the user in the security database.
+#ifdef REEXPAND_DBNAME
+		// Needed to ensure correct locking order with security database mutex
+		MutexLockGuard dbInitGuard(JRD_get_dbinitmutex());
+#endif
+
 		SecurityDatabase::InitHolder siHolder;
+
+		// Perhaps checkout the user in the security database.
 		USHORT user_flag;
 		if (!strcmp(serv->serv_name, "anonymous")) {
 			user_flag = SVC_user_none;
@@ -1329,7 +1350,7 @@ ISC_STATUS Service::query2(thread_db* tdbb,
 					gds__prefix(auxBuf, "");
 					break;
 				case isc_info_svc_get_env_lock:
-					gds__prefix_lock(auxBuf, "");
+					iscPrefixLock(auxBuf, "", false);
 					break;
 				case isc_info_svc_get_env_msg:
 					gds__prefix_msg(auxBuf, "");
@@ -1813,7 +1834,7 @@ void Service::query(USHORT			send_item_length,
 					gds__prefix(PathBuffer, "");
 					break;
 				case isc_info_svc_get_env_lock:
-					gds__prefix_lock(PathBuffer, "");
+					iscPrefixLock(PathBuffer, "", false);
 					break;
 				case isc_info_svc_get_env_msg:
 					gds__prefix_msg(PathBuffer, "");
@@ -2166,7 +2187,8 @@ void Service::start(USHORT spb_length, const UCHAR* spb_data)
 		svc_id == isc_action_svc_trace_resume ||
 		svc_id == isc_action_svc_trace_list ||
 		svc_id == isc_action_svc_set_mapping ||
-		svc_id == isc_action_svc_drop_mapping)
+		svc_id == isc_action_svc_drop_mapping ||
+		svc_id == isc_action_svc_validate)
 	{
 		/* add the username and password to the end of svc_switches if needed */
 		if (svc_switches.hasData())
@@ -2709,6 +2731,8 @@ bool Service::process_switches(ClumpletReader& spb, string& switches)
 	string nbk_database, nbk_file;
 	int nbk_level = -1;
 
+	bool val_database = false;
+
 	bool found = false;
 
 	do
@@ -2986,6 +3010,13 @@ bool Service::process_switches(ClumpletReader& spb, string& switches)
 				}
 				get_action_svc_string(spb, switches);
 				break;
+			case isc_spb_bkp_stat:
+				if (!get_action_svc_parameter(spb.getClumpTag(), reference_burp_in_sw_table, switches))
+				{
+					return false;
+				}
+				get_action_svc_string(spb, switches);
+				break;
 			default:
 				return false;
 			}
@@ -3083,6 +3114,31 @@ bool Service::process_switches(ClumpletReader& spb, string& switches)
 			}
 			break;
 
+		case isc_action_svc_validate:
+			if (!get_action_svc_parameter(spb.getClumpTag(), val_option_in_sw_table, switches)) {
+				return false;
+			}
+
+			switch (spb.getClumpTag())
+			{
+			case isc_spb_dbname:
+				if (val_database) {
+					(Arg::Gds(isc_unexp_spb_form) << Arg::Str("only one isc_spb_dbname")).raise();
+				}
+				val_database = true;
+				// fall thru
+			case isc_spb_val_tab_incl:
+			case isc_spb_val_tab_excl:
+			case isc_spb_val_idx_incl:
+			case isc_spb_val_idx_excl:
+				get_action_svc_string(spb, switches);
+				break;
+			case isc_spb_val_lock_timeout:
+				get_action_svc_data(spb, switches);
+				break;
+			}
+			break;
+
 		default:
 			return false;
 		}
@@ -3133,6 +3189,13 @@ bool Service::process_switches(ClumpletReader& spb, string& switches)
 		switches += nbk_database;
 		switches += nbk_file;
 		break;
+	
+	case isc_action_svc_validate:
+		if (!val_database)
+		{
+			(Arg::Gds(isc_missing_required_spb) << Arg::Str("isc_spb_dbname")).raise();
+		}
+		break;
 	}
 
 	switches.rtrim();
@@ -3179,7 +3242,7 @@ void Service::get_action_svc_data(const ClumpletReader& spb,
 								  string& switches)
 {
 	string s;
-	s.printf("%"ULONGFORMAT" ", spb.getInt());
+	s.printf("%" SLONGFORMAT" ", spb.getInt());
 	switches += s;
 }
 
