@@ -31,6 +31,7 @@
 #include "../jrd/CharSet.h"
 #include "../jrd/IntlUtil.h"
 #include "../jrd/gdsassert.h"
+#include "../jrd/os/path_utils.h"
 #include "../common/classes/auto.h"
 #include "../common/classes/GenericMap.h"
 #include "../common/classes/init.h"
@@ -42,6 +43,9 @@
 #include "unicode/ucnv.h"
 #include "unicode/ucol.h"
 
+// The next major ICU version after 4.8 is 49.
+#define ICU_NEW_VERSION_MEANING	49
+
 
 using namespace Firebird;
 
@@ -49,10 +53,19 @@ using namespace Firebird;
 namespace Jrd {
 
 
+#if U_ICU_VERSION_MAJOR_NUM >= ICU_NEW_VERSION_MEANING
+const char* const UnicodeUtil::DEFAULT_ICU_VERSION = STRINGIZE(U_ICU_VERSION_MAJOR_NUM);
+#else
 const char* const UnicodeUtil::DEFAULT_ICU_VERSION =
-	STRINGIZE(U_ICU_VERSION_MAJOR_NUM)"."STRINGIZE(U_ICU_VERSION_MINOR_NUM);
+	STRINGIZE(U_ICU_VERSION_MAJOR_NUM)"." STRINGIZE(U_ICU_VERSION_MINOR_NUM);
+#endif
 
 
+static void formatFilename(PathName& filename, const char* templateName,
+	int majorVersion, int minorVersion);
+
+static ModuleLoader::Module* tryLoadIcuModule(const PathName& filename);
+
 // encapsulate ICU collations libraries
 struct UnicodeUtil::ICU
 {
@@ -65,12 +78,16 @@ public:
 		: majorVersion(aMajorVersion),
 		  minorVersion(aMinorVersion),
 		  inModule(NULL),
-		  ucModule(NULL)
+		  ucModule(NULL),
+		  ciAiTransCache(*getDefaultMemoryPool())
 	{
 	}
 
 	~ICU()
 	{
+		while (ciAiTransCache.hasData())
+			utransClose(ciAiTransCache.pop());
+
 		delete ucModule;
 		delete inModule;
 	}
@@ -79,6 +96,11 @@ public:
 	{
 		string symbol;
 
+		symbol.printf("%s_%d", name, majorVersion);
+		module->findSymbol(symbol, ptr);
+		if (ptr)
+			return;
+
 		symbol.printf("%s_%d_%d", name, majorVersion, minorVersion);
 		module->findSymbol(symbol, ptr);
 		if (ptr)
@@ -86,6 +108,40 @@ public:
 
 		symbol.printf("%s_%d%d", name, majorVersion, minorVersion);
 		module->findSymbol(symbol, ptr);
+		if (ptr)
+			return;
+
+		symbol.printf("%s", name);
+		module->findSymbol(symbol, ptr);
+	}
+
+	UTransliterator* getCiAiTransliterator()
+	{
+		ciAiTransCacheMutex.enter();
+		UTransliterator* ret;
+
+		if (!ciAiTransCache.isEmpty())
+		{
+			ret = ciAiTransCache.pop();
+			ciAiTransCacheMutex.leave();
+		}
+		else
+		{
+			ciAiTransCacheMutex.leave();
+
+			UErrorCode errorCode = U_ZERO_ERROR;
+			// Fix for CORE-4136. Was "Any-Upper; NFD; [:Nonspacing Mark:] Remove; NFC".
+			ret = utransOpen("NFD; [:Nonspacing Mark:] Remove; NFC",
+				UTRANS_FORWARD, NULL, 0, NULL, &errorCode);
+		}
+
+		return ret;
+	}
+
+	void releaseCiAiTransliterator(UTransliterator* trans)
+	{
+		MutexLockGuard guard(ciAiTransCacheMutex);
+		ciAiTransCache.push(trans);
 	}
 
 	int majorVersion;
@@ -93,6 +149,8 @@ public:
 	ModuleLoader::Module* inModule;
 	ModuleLoader::Module* ucModule;
 	UVersionInfo collVersion;
+	Mutex ciAiTransCacheMutex;
+	Array<UTransliterator*> ciAiTransCache;
 
 	void (U_EXPORT2 *uInit)(UErrorCode* status);
 	void (U_EXPORT2 *uVersionToString)(UVersionInfo versionArray, char* versionString);
@@ -157,14 +215,25 @@ public:
 	RWLock lock;
 };
 
-namespace {
-	GlobalPtr<UnicodeUtil::ICUModules> icuModules;
-}
+static GlobalPtr<UnicodeUtil::ICUModules> icuModules;
 
 
 static const char* const COLL_30_VERSION = "41.128.4.4";	// ICU 3.0 collator version
 
 
+static void formatFilename(PathName& filename, const char* templateName,
+	int majorVersion, int minorVersion)
+{
+	string s;
+	if (majorVersion >= ICU_NEW_VERSION_MEANING)
+		s.printf("%d", majorVersion);
+	else
+		s.printf("%d%d", majorVersion, minorVersion);
+
+	filename.printf(templateName, s.c_str());
+}
+
+
 static void getVersions(const string& configInfo, ObjectsArray<string>& versions)
 {
 	charset cs;
@@ -201,6 +270,21 @@ static void getVersions(const string& configInfo, ObjectsArray<string>& versions
 }
 
 
+static ModuleLoader::Module* tryLoadIcuModule(const PathName& filename)
+{
+	ModuleLoader::Module* module = ModuleLoader::loadModule(filename);
+
+	if (!module)
+	{
+		PathName name(filename);
+		ModuleLoader::doctorModuleExtention(name);
+		module = ModuleLoader::loadModule(name);
+	}
+
+	return module;
+}
+
+
 // BOCU-1
 USHORT UnicodeUtil::utf16KeyLength(USHORT len)
 {
@@ -758,17 +842,17 @@ UnicodeUtil::ICU* UnicodeUtil::loadICU(const Firebird::string& icuVersion,
 	const Firebird::string& configInfo)
 {
 #if defined(WIN_NT)
-	const char* const inTemplate = "icuin%d%d.dll";
-	const char* const ucTemplate = "icuuc%d%d.dll";
+	const char* const inTemplate = "icuin%s.dll";
+	const char* const ucTemplate = "icuuc%s.dll";
 #elif defined(DARWIN)
 	const char* const inTemplate = "/Library/Frameworks/Firebird.framework/Versions/A/Libraries/libicui18n.dylib";
 	const char* const ucTemplate = "/Library/Frameworks/Firebird.framework/versions/A/Libraries/libicuuc.dylib";
 #elif defined(HPUX)
-	const char* const inTemplate = "libicui18n.sl.%d%d";
-	const char* const ucTemplate = "libicuuc.sl.%d%d";
+	const char* const inTemplate = "libicui18n.sl.%s";
+	const char* const ucTemplate = "libicuuc.sl.%s";
 #else
-	const char* const inTemplate = "libicui18n.so.%d%d";
-	const char* const ucTemplate = "libicuuc.so.%d%d";
+	const char* const inTemplate = "libicui18n.so.%s";
+	const char* const ucTemplate = "libicuuc.so.%s";
 #endif
 
 	ObjectsArray<string> versions;
@@ -776,24 +860,28 @@ UnicodeUtil::ICU* UnicodeUtil::loadICU(const Firebird::string& icuVersion,
 
 	string version = icuVersion.isEmpty() ? versions[0] : icuVersion;
 	if (version == "default")
-	{
-		version.printf("%d.%d", U_ICU_VERSION_MAJOR_NUM, U_ICU_VERSION_MINOR_NUM);
-	}
+		version = DEFAULT_ICU_VERSION;
 
 	for (ObjectsArray<string>::const_iterator i(versions.begin()); i != versions.end(); ++i)
 	{
 		int majorVersion, minorVersion;
+		int n = sscanf((*i == "default" ? version : *i).c_str(), "%d.%d",
+			&majorVersion, &minorVersion);
 
-		if (*i == "default")
-		{
-			majorVersion = U_ICU_VERSION_MAJOR_NUM;
-			minorVersion = U_ICU_VERSION_MINOR_NUM;
-		}
-		else if (sscanf(i->c_str(), "%d.%d", &majorVersion, &minorVersion) != 2)
+		if (n == 1)
+			minorVersion = 0;
+		else if (n != 2)
 			continue;
 
 		string configVersion;
-		configVersion.printf("%d.%d", majorVersion, minorVersion);
+
+		if (majorVersion >= ICU_NEW_VERSION_MEANING)
+		{
+			minorVersion = 0;
+			configVersion.printf("%d", majorVersion);
+		}
+		else
+			configVersion.printf("%d.%d", majorVersion, minorVersion);
 
 		if (version != configVersion)
 			continue;
@@ -805,16 +893,11 @@ UnicodeUtil::ICU* UnicodeUtil::loadICU(const Firebird::string& icuVersion,
 			return icu;
 
 		PathName filename;
-		filename.printf(ucTemplate, majorVersion, minorVersion);
+		formatFilename(filename, ucTemplate, majorVersion, minorVersion);
 
 		icu = FB_NEW(*getDefaultMemoryPool()) ICU(majorVersion, minorVersion);
 
-		icu->ucModule = ModuleLoader::loadModule(filename);
-		if (!icu->ucModule)
-		{
-			ModuleLoader::doctorModuleExtention(filename);
-			icu->ucModule = ModuleLoader::loadModule(filename);
-		}
+		icu->ucModule = tryLoadIcuModule(filename);
 
 		if (!icu->ucModule)
 		{
@@ -822,14 +905,25 @@ UnicodeUtil::ICU* UnicodeUtil::loadICU(const Firebird::string& icuVersion,
 			continue;
 		}
 
-		filename.printf(inTemplate, majorVersion, minorVersion);
-
-		icu->inModule = ModuleLoader::loadModule(filename);
-		if (!icu->inModule)
+		formatFilename(filename, inTemplate, majorVersion, minorVersion);
+#ifdef WIN_NT
+		// Try to load inModule from the same folder as ucModule
 		{
-			ModuleLoader::doctorModuleExtention(filename);
-			icu->inModule = ModuleLoader::loadModule(filename);
+			PathName ucPath;
+			if (icu->ucModule->getModuleFileName(ucPath))
+			{
+				size_t pos = ucPath.rfind(PathUtils::dir_sep);
+				if (pos != PathName::npos)
+				{
+					ucPath.resize(pos + 1);
+					ucPath.append(filename);
+					icu->inModule = tryLoadIcuModule(ucPath);
+				}
+			}
 		}
+		if (!icu->inModule)
+#endif
+		icu->inModule = tryLoadIcuModule(filename);
 
 		if (!icu->inModule)
 		{
@@ -1208,44 +1302,40 @@ SSHORT UnicodeUtil::Utf16Collation::compare(ULONG len1, const USHORT* str1,
 ULONG UnicodeUtil::Utf16Collation::canonical(ULONG srcLen, const USHORT* src, ULONG dstLen, ULONG* dst,
 	const ULONG* exceptions)
 {
+	fb_assert(srcLen % sizeof(*src) == 0);
+
 	HalfStaticArray<USHORT, BUFFER_SMALL / 2> upperStr;
 
-	if ((attributes & (TEXTTYPE_ATTR_CASE_INSENSITIVE | TEXTTYPE_ATTR_ACCENT_INSENSITIVE)) ==
-		(TEXTTYPE_ATTR_CASE_INSENSITIVE | TEXTTYPE_ATTR_ACCENT_INSENSITIVE))
+	if (attributes & TEXTTYPE_ATTR_CASE_INSENSITIVE)
 	{
-		fb_assert(srcLen % sizeof(*src) == 0);
-
-		memcpy(upperStr.getBuffer(srcLen / sizeof(USHORT)), src, srcLen);
-
-		UErrorCode errorCode = U_ZERO_ERROR;
-		UTransliterator* trans = icu->utransOpen("Any-Upper; NFD; [:Nonspacing Mark:] Remove; NFC",
-			UTRANS_FORWARD, NULL, 0, NULL, &errorCode);
-
-		if (errorCode <= 0)
-		{
-			const int32_t capacity = upperStr.getCount();
-			int32_t len = srcLen / sizeof(USHORT);
-			int32_t limit = len;
-
-			icu->utransTransUChars(trans, reinterpret_cast<UChar*>(upperStr.begin()),
-				&len, capacity, 0, &limit, &errorCode);
-			icu->utransClose(trans);
-
-			len *= sizeof(USHORT);
-			if (ULONG(len) > dstLen)
-				len = INTL_BAD_STR_LENGTH;
-
-			srcLen = len;
-			src = upperStr.begin();
-		}
-		else
-			return INTL_BAD_STR_LENGTH;
-	}
-	else if (attributes & TEXTTYPE_ATTR_CASE_INSENSITIVE)
-	{
-		srcLen = utf16UpperCase(srcLen, src,
-			srcLen, upperStr.getBuffer(srcLen / sizeof(USHORT)), exceptions);
+		srcLen = utf16UpperCase(srcLen, src, srcLen,
+			upperStr.getBuffer(srcLen / sizeof(USHORT)), NULL);
 		src = upperStr.begin();
+
+		if (attributes & TEXTTYPE_ATTR_ACCENT_INSENSITIVE)
+		{
+			UTransliterator* trans = icu->getCiAiTransliterator();
+
+			if (trans)
+			{
+				const int32_t capacity = upperStr.getCount();
+				int32_t len = srcLen / sizeof(USHORT);
+				int32_t limit = len;
+
+				UErrorCode errorCode = U_ZERO_ERROR;
+				icu->utransTransUChars(trans, reinterpret_cast<UChar*>(upperStr.begin()),
+					&len, capacity, 0, &limit, &errorCode);
+				icu->releaseCiAiTransliterator(trans);
+
+				len *= sizeof(USHORT);
+				if (ULONG(len) > dstLen)
+					len = INTL_BAD_STR_LENGTH;
+
+				srcLen = len;
+			}
+			else
+				return INTL_BAD_STR_LENGTH;
+		}
 	}
 
 	// convert UTF-16 to UTF-32
