@@ -159,11 +159,6 @@ static void SOCLOSE(SOCKET& socket)
 	}
 };
 
-// Can't find were it's used.
-//#ifndef SIGURG
-//#define SIGURG	SIGINT
-//#endif
-
 #ifndef ENOBUFS
 #define ENOBUFS	0
 #endif
@@ -437,6 +432,8 @@ static int		cleanup_ports(const int, const int, void*);
 static int		fork();
 #endif
 
+typedef Firebird::Array<SOCKET> SocketsArray;
+
 #ifdef WIN_NT
 static void		wsaExitHandler(void*);
 static int		fork(SOCKET, USHORT);
@@ -446,9 +443,7 @@ static Firebird::GlobalPtr<Firebird::Mutex> forkMutex;
 static HANDLE forkEvent = INVALID_HANDLE_VALUE;
 static bool forkThreadStarted = false;
 
-typedef Firebird::Array<SOCKET> SocketsArray;
 static SocketsArray* forkSockets;
-
 #endif
 
 static in_addr get_bind_address();
@@ -549,7 +544,7 @@ static rem_port* inet_async_receive = NULL;
 
 static Firebird::GlobalPtr<Firebird::Mutex> port_mutex;
 static Firebird::GlobalPtr<PortsCleanup>	inet_ports;
-
+static Firebird::GlobalPtr<SocketsArray>	ports_to_close;
 
 rem_port* INET_analyze(const Firebird::PathName& file_name,
 					ISC_STATUS*	status_vector,
@@ -1076,10 +1071,23 @@ rem_port* INET_connect(const TEXT* name,
 			}
 			return NULL;
 		}
+
 #ifdef WIN_NT
 		if (flag & SRVR_debug)
 #else
-		if ((flag & SRVR_debug) || !fork())
+		int pid = 0;
+		if (!(flag & SRVR_debug))
+		{
+			pid = fork();
+			if (pid < 0)
+			{
+				inet_error(port, "fork", isc_net_connect_err, inetErrNo);
+				disconnect(port);
+				return NULL;
+			}
+		}
+
+		if (!pid)
 #endif
 		{
 			SOCLOSE(port->port_handle);
@@ -1736,13 +1744,29 @@ static void disconnect(rem_port* const port)
 		port->port_async = NULL;
 	}
 
+	// hvlad: delay closing of the server sockets to prevent its reuse
+	// by another (newly accepted) port until next select() call. See
+	// also select_wait() function.
+	const bool delayClose = (port->port_server_flags && port->port_parent);
+
 	// If this is a sub-port, unlink it from its parent
 	port->unlinkParent();
 
 	inet_ports->unRegisterPort(port);
 
-	SOCLOSE(port->port_handle);
-	SOCLOSE(port->port_channel);
+	if (delayClose)
+	{
+		if (port->port_handle != INVALID_SOCKET)
+			ports_to_close->push(port->port_handle);
+
+		if (port->port_channel != INVALID_SOCKET)
+			ports_to_close->push(port->port_channel);
+	}
+	else
+	{
+		SOCLOSE(port->port_handle);
+		SOCLOSE(port->port_channel);
+	}
 
 	port->release();
 
@@ -1803,6 +1827,13 @@ static int cleanup_ports(const int, const int, void* /*arg*/)
 	INET_shutting_down = true;
 
 	inet_ports->closePorts();
+
+	while (ports_to_close->hasData())
+	{
+		SOCKET s = ports_to_close->pop();
+		SOCLOSE(s);
+	}
+
 	return 0;
 }
 
@@ -1868,7 +1899,7 @@ static int fork(SOCKET old_handle, USHORT flag)
 	}
 
 	Firebird::string cmdLine;
-	cmdLine.printf("%s -i -h %"HANDLEFORMAT"@%"ULONGFORMAT, name, new_handle, GetCurrentProcessId());
+	cmdLine.printf("%s -i -h %" HANDLEFORMAT"@%" ULONGFORMAT, name, new_handle, GetCurrentProcessId());
 
 	STARTUPINFO start_crud;
 	start_crud.cb = sizeof(STARTUPINFO);
@@ -2324,6 +2355,13 @@ static bool select_wait( rem_port* main_port, Select* selct)
 
 		{ // port_mutex scope
 			Firebird::MutexLockGuard guard(port_mutex);
+
+			while (ports_to_close->hasData())
+			{
+				SOCKET s = ports_to_close->pop();
+				SOCLOSE(s);
+			}
+
 			for (rem_port* port = main_port; port; port = port->port_next)
 			{
 				if (port->port_state == rem_port::PENDING && 
@@ -2358,7 +2396,7 @@ static bool select_wait( rem_port* main_port, Select* selct)
 							if (badSocket || INET_ERRNO == NOTASOCKET)
 							{
 								// not a socket, strange !
-								gds__log("INET/select_wait: found \"not a socket\" socket : %"HANDLEFORMAT,
+								gds__log("INET/select_wait: found \"not a socket\" socket : %" HANDLEFORMAT,
 										 port->port_handle);
 
 								// this will lead to receive() which will break bad connection
@@ -2611,7 +2649,7 @@ static bool_t inet_getbytes( XDR* xdrs, SCHAR* buff, u_int count)
  **************************************/
 #ifdef REM_SERVER
 	const rem_port* port = (rem_port*) xdrs->x_public;
-	if ((port->port_flags & PORT_server) && !(port->port_server_flags & SRVR_debug))
+	if (port->port_flags & PORT_server)
 	{
 		return REMOTE_getbytes(xdrs, buff, count);
 	}
@@ -3124,6 +3162,13 @@ static bool packet_receive(rem_port* port, UCHAR* buffer, SSHORT buffer_length,
 	const timeval savetime = timeout;
 
 	const SOCKET ph = port->port_handle;
+	if (ph == INVALID_SOCKET)
+	{
+		if (!(port->port_flags & PORT_disconnect)) {
+			inet_error(port, "invalid socket in packet_receive", isc_net_read_err, EINVAL);
+		}
+		return false;
+	}
 
 	// Unsed to send a dummy packet, but too big to be defined in the loop.
 	PACKET packet;
