@@ -239,7 +239,14 @@ bool putNumericArgument(char**& av, ClumpletWriter& spb, unsigned int tag)
 	if (! *av)
 		return false;
 
-	int n = atoi(*av++);
+	char* err = NULL;
+	SLONG n = strtol(*av++, &err, 10);
+
+	if (err && *err)
+	{
+		(Arg::Gds(isc_fbsvcmgr_bad_arg) << av[-2]).raise();
+	}
+
 	spb.insertInt(tag, n);
 
 	return true;
@@ -338,6 +345,7 @@ const SvcSwitches backupOptions[] =
 	{"bkp_non_transportable", putOption, 0, isc_spb_bkp_non_transportable, 0},
 	{"bkp_convert", putOption, 0, isc_spb_bkp_convert, 0},
 	{"bkp_no_triggers", putOption, 0, isc_spb_bkp_no_triggers, 0},
+	{"bkp_stat", putStringArgument, 0, isc_spb_bkp_stat, 0 },
 	{0, 0, 0, 0, 0}
 };
 
@@ -360,6 +368,7 @@ const SvcSwitches restoreOptions[] =
 	{"res_fix_fss_data", putStringArgument, 0, isc_spb_res_fix_fss_data, 0},
 	{"res_fix_fss_metadata", putStringArgument, 0, isc_spb_res_fix_fss_metadata, 0},
 	{"res_metadata_only", putOption, 0, isc_spb_res_metadata_only, 0},
+	{"res_stat", putStringArgument, 0, isc_spb_res_stat, 0 },
 	{0, 0, 0, 0, 0}
 };
 
@@ -471,11 +480,21 @@ const SvcSwitches traceStartOptions[] =
 
 const SvcSwitches traceChgStateOptions[] =
 {
-	{"trc_name", putStringArgument, 0, isc_spb_trc_name, 0},
 	{"trc_id", putNumericArgument, 0, isc_spb_trc_id, 0},
 	{0, 0, 0, 0, 0}
 };
 
+const SvcSwitches validateOptions[] = 
+{
+	{"dbname", putStringArgument, 0, isc_spb_dbname, 0},
+	{"val_tab_incl", putStringArgument, 0, isc_spb_val_tab_incl, 0},
+	{"val_tab_excl", putStringArgument, 0, isc_spb_val_tab_excl, 0},
+	{"val_idx_incl", putStringArgument, 0, isc_spb_val_idx_incl, 0},
+	{"val_idx_excl", putStringArgument, 0, isc_spb_val_idx_excl, 0},
+	{"val_lock_timeout", putNumericArgument, 0, isc_spb_val_lock_timeout, 0},
+	{0, 0, 0, 0, 0}
+};
+
 const SvcSwitches actionSwitch[] =
 {
 	{"action_backup", putSingleTag, backupOptions, isc_action_svc_backup, isc_info_svc_to_eof},
@@ -498,6 +517,7 @@ const SvcSwitches actionSwitch[] =
 	{"action_trace_list", putSingleTag, 0, isc_action_svc_trace_list, isc_info_svc_line},
 	{"action_set_mapping", putSingleTag, mappingOptions, isc_action_svc_set_mapping, 0},
 	{"action_drop_mapping", putSingleTag, mappingOptions, isc_action_svc_drop_mapping, 0},
+	{"action_validate", putSingleTag, validateOptions, isc_action_svc_validate, isc_info_svc_line},
 	{0, 0, 0, 0, 0}
 };
 
@@ -846,6 +866,23 @@ static int shutdownCallback(const int reason, const int, void*)
 	return FB_SUCCESS;
 }
 
+class ShutdownHolder
+{
+public:
+	bool active;
+
+	ShutdownHolder()
+		: active(false)
+	{ }
+
+	~ShutdownHolder()
+	{
+		if (active)
+		{
+			fb_shutdown(0, fb_shutrsn_exit_called);
+		}
+	}
+};
 
 // simple main function
 
@@ -866,6 +903,8 @@ int main(int ac, char** av)
 	prevCtrlCHandler = signal(SIGINT, ctrl_c_handler);
 	fb_shutdown_callback(NULL, shutdownCallback, fb_shut_confirmation, NULL);
 
+	ShutdownHolder shutdownHolder;
+
 	ISC_STATUS_ARRAY status;
 
 	try {
@@ -931,6 +970,8 @@ int main(int ac, char** av)
 			return 1;
 		}
 
+		shutdownHolder.active = true;
+
 		if (spbStart.getBufferLength() > 0)
 		{
 			if (isc_service_start(status, &svc_handle, 0,
